﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;

namespace HeuristicLab.BioBoost.Operators.Mutation {
  [StorableClass]
  public class BoundaryToggleRealVectorMutator : RealVectorManipulator {

    #region Construction & Cloning
    [StorableConstructor]
    protected BoundaryToggleRealVectorMutator(bool isDeserializing) : base(isDeserializing) { }
    protected BoundaryToggleRealVectorMutator(BoundaryToggleRealVectorMutator orig, Cloner cloner) : base(orig, cloner) { }
    public BoundaryToggleRealVectorMutator() { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new BoundaryToggleRealVectorMutator(this, cloner);
    }
    #endregion

    protected override void Manipulate(IRandom random, RealVector realVector) {
      var bounds = BoundsParameter.ActualValue;
      var idx = random.Next(realVector.Length);
      var value = realVector[idx];
      var lowerBound = bounds[idx % bounds.Rows, 0];
      var upperBound = bounds[idx % bounds.Rows, 1];
      if (value <= lowerBound || value >= upperBound) {
        value = random.Next()*Math.Abs(upperBound - lowerBound) + Math.Min(lowerBound, upperBound);
      } else {
        value = random.NextDouble() <= 0.5 ? lowerBound : upperBound;
      }
      realVector[idx] = value;
    }
  }
}
